from typing import List, Optional

from appium.webdriver.appium_service import AppiumService
from behave.configuration import Configuration as BehaveConfiguration
from behave.model import Feature, Tag, Scenario, Table, Row
from selenium.webdriver.remote.webdriver import WebDriver

from rikki.appium.common import AppiumUtils
from rikki.mitmproxy.plugin import ProxyPlugin


class Configuration(BehaveConfiguration):

    def __init__(self, command_args=None, load_config=True, verbose=None, **kwargs):
        super().__init__(command_args, load_config, verbose, **kwargs)
        self.proxy: Optional[ProxyPlugin] = None


class Context:
    """
    Provide typed access to Behave context object.
    More documentation can'be founded there https://behave.readthedocs.io/en/latest/context_attributes.html
    """

    def __init__(
            self,
            config: Optional[Configuration] = None,
            aborted: Optional[bool] = None,
            failed: Optional[bool] = None,
            feature: Optional[Feature] = None,
            tags: Optional[List[Tag]] = None,
            active_outline: Optional[Row] = None,
            scenario: Optional[Scenario] = None,
            table: Optional[Table] = None,
            text: Optional[str] = None,
            proxy: Optional[ProxyPlugin] = None,
            browser: Optional[WebDriver] = None,
            appium: Optional[AppiumService] = None,
            appium_utils: Optional[AppiumUtils] = None
    ) -> None:
        super().__init__()
        self.config: Optional[Configuration] = config
        self.aborted: Optional[bool] = aborted
        self.failed: Optional[bool] = failed
        self.feature: Optional[Feature] = feature
        self.tags: Optional[List[Tag]] = tags
        self.active_outline: Optional[Row] = active_outline
        self.scenario: Optional[Scenario] = scenario
        self.table: Optional[Table] = table
        self.text: Optional[str] = text
        self.proxy: Optional[ProxyPlugin] = proxy
        self.browser: Optional[WebDriver] = browser
        self.appium: Optional[AppiumService] = appium
        self.appium_utils: Optional[AppiumUtils] = appium_utils



from mitmproxy import http
from rikki.mitmproxy.model import Request, Response
from typing import List, Optional, Dict
from mitmproxy.coretypes import multidict
from mitmproxy.net.http import Headers


def replace(flow: http.HTTPFlow, request: Optional[Request] = None, response: Optional[Response] = None):
    if request:
        if request.host:
            flow.request.host = request.host
        if request.content:
            flow.request.content = request.content.encode("utf8")
        if request.params:
            for item in request.params.items():
                flow.request.query.set_all(item[0], [item[1]])
        if request.headers:
            for item in request.headers.items():
                headers: Headers = flow.request.headers
                headers.set_all(item[0], [item[1]])
    if response:
        if response.headers:
            for item in response.headers.items():
                headers: Headers = flow.response.headers
                headers.set_all(item[0], [item[1]])
        if response.content:
            flow.response.content = response.content.encode("utf8")
        if response.code:
            flow.response.status_code = response.code


def filter(
        flows: List[http.HTTPFlow],
        request: Optional[Request] = None,
        response: Optional[Response] = None
) -> List[http.HTTPFlow]:
    """
    Filter flows and return only those that contain data from filter params
    :param flows: list of flows to be filtered
    :param request: request data to look for during filter
    :param response: response data to look for during filter
    :return: new list of filtered flows
    """
    result: List[http.HTTPFlow] = []
    result.extend(flows)

    for flow in result:
        if request:
            if request.host and request.host != flow.request.host:
                if flow in result:
                    result.remove(flow)
                continue
            if request.params and not contains_params(flow.request.query, request.params):
                if flow in result:
                    result.remove(flow)
                continue
            if request.content and not request.content.lower().strip() != flow.request.content.lower().strip():
                if flow in result:
                    result.remove(flow)
            if request.headers and not contains_headers(flow.request.headers, request.headers):
                if flow in result:
                    result.remove(flow)
        if response:
            if response.code and response.code != flow.response.status_code:
                if flow in result:
                    result.remove(flow)
            if response.headers and not contains_headers(flow.response.headers, response.headers):
                if flow in result:
                    result.remove(flow)
            if response.content and flow.response.content \
                    and response.content.lower().strip() != flow.response.content.decode("utf8").lower().strip():
                if flow in result:
                    result.remove(flow)

    return result


def contains_params(base: multidict.MultiDictView, candidate: Dict) -> bool:
    for key in candidate.keys():
        if key in base.keys() and candidate[key] in base.get_all(key):
            continue
        else:
            return False
    return True


def contains_headers(headers: Headers, candidate: Dict[str, str]):
    for key in candidate.keys():
        if key in headers.keys() and candidate[key] in headers.get_all(key):
            continue
        else:
            return False
    return True



import sys
from typing import Optional, Sequence
from rikki.template.template import behave_init
from rikki.run import RikkiTestRunner


def rikki(args: Optional[Sequence[str]] = sys.argv[1:]) -> Optional[int]:
    if not args:
        print("Use --help to find possible options")
        return
    if len(args) > 1:
        print("You can't use more than 1 argument. Use --help to find possible options")
        return

    command = args[0]

    if command == "init":
        behave_init()
        return

    if command == "run":
        RikkiTestRunner().run()
        return

    if command == "--help":
        print("init - will init folder structures for behave test environment")
        print("run - will run tests defined in the folder with default config")
        print("--help - show this help")


if __name__ == "__main__":
    rikki(sys.argv[1:])



import time
from unittest import TestCase

from rikki.mitmproxy.delegate_delay import DelayConfig, Delay
from rikki.mitmproxy.model import Request
from test.rikki.mitmproxy.utils import build_flow


class TestDelay(TestCase):

    def test_delay(self):
        request = Request(host="test_host")

        config = DelayConfig(request=request, delay=1)

        instance = Delay(config=config)

        flow = build_flow(request=request)

        instance.response(flow)

        assert flow.intercepted, "The flow hasn't been intercepted"

        time.sleep(4)

        assert not flow.intercepted, "The flow hasn't been resumed"



from enum import Enum
from typing import Optional

from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.support import expected_conditions as conditions
from selenium.webdriver.support.wait import WebDriverWait


class Direction(Enum):
    UP = 1
    DOWN = 2
    LEFT = 3
    RIGHT = 4


class AppiumUtils:

    def __init__(self, driver: WebDriver, wait_time: int = 1) -> None:
        super().__init__()
        self.wait_time = wait_time
        self._driver: WebDriver = driver

    def tap(self, driver: WebDriver, by: By, locator: str, wait_time: Optional[int] = None):
        wait = self._configure_wait(driver, wait_time)
        wait.until(conditions.presence_of_element_located((by, locator))).click()

    def enter(self, driver: WebDriver, by: By, locator: str, text: str, wait_time: Optional[int] = None):
        wait = self._configure_wait(driver, wait_time)
        wait.until(conditions.presence_of_element_located((by, locator))).send_keys(text)

    def wait(self, driver: WebDriver, by: By, locator: str, wait_time: Optional[int] = None):
        wait = self._configure_wait(driver, wait_time)
        wait.until(by, locator)

    def swipe(self, direction: Direction):
        assert False, "Swipe is not supported:"

    def _configure_wait(self, driver: WebDriver, wait_time: Optional[int] = None):
        delay = self.wait_time
        if wait_time:
            delay = wait_time
        return WebDriverWait(driver, delay)

    def support(self, driver: WebDriver) -> bool:
        """
        :param driver:
        :return: True if this instance of the utils support the context
        """
        return True



import threading

from behave.__main__ import run_behave
from behave.configuration import Configuration
from mitmproxy import options
from mitmproxy.tools.dump import DumpMaster
from mitmproxy.tools.main import run as run_mitmproxy, cmdline

from rikki.mitmproxy.plugin import ProxyPlugin


class RikkiConfiguration:

    def __init__(self) -> None:
        super().__init__()
        self.proxy_plugin = ProxyPlugin()


config = RikkiConfiguration()


class ExtendedDumpMaster(DumpMaster):

    def __init__(
            self,
            options: options.Options,
            with_termlog=False,
            with_dumper=True
    ) -> None:
        super().__init__(options, with_termlog, with_dumper)
        self.addons.add(*[config.proxy_plugin])
        options.flow_detail = 0
        config.proxy_plugin.proxy = self


class RikkiTestRunner:

    def run(self, plugin: ProxyPlugin = ProxyPlugin()):
        config.proxy_plugin = plugin
        self._configure()

    def _configure(self):
        behave_thread = threading.Thread(target=self._start_behave)
        behave_thread.start()
        run_mitmproxy(ExtendedDumpMaster, cmdline.mitmdump, None, {})

    def _start_behave(self):
        configuration = Configuration(stdout_capture=False, stderr_capture=False, log_capture=False, junit=True)
        configuration.mitm_plugin = config.proxy_plugin
        configuration.proxy = config.proxy_plugin
        run_behave(configuration)



from selenium.webdriver.remote.webdriver import WebDriver

from rikki.appium.common import AppiumUtils, Direction


class AppiumAndroidEspressoUtils(AppiumUtils):

    def swipe(self, direction: Direction):
        if direction == Direction.UP:
            self._driver.execute_script('mobile: swipe', {'element': 2, 'direction': 'up'})
        if direction == Direction.DOWN:
            self._driver.execute_script('mobile: swipe', {'element': 2, 'direction': 'down'})
        if direction == Direction.LEFT or direction == Direction.RIGHT:
            assert False, "Directions START and END are not supported yet"

    def support(self, driver: WebDriver) -> bool:
        platform = driver.desired_capabilities['platformName']
        automation = driver.desired_capabilities['automationName']

        return platform == "Android" and automation == "Espresso"





import os
import sys

import setuptools
from setuptools.command.install import install


def version() -> str: return "0.0.24"


class VerifyVersionCommand(install):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'

    def run(self):
        tag = os.getenv('CIRCLE_TAG')

        if tag != version():
            info = "Git tag: {0} does not match the version of this app: {1}".format(
                tag, version()
            )
            sys.exit(info)


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rikki",
    version=version(),
    author="Sergey Yamshchikov",
    author_email="yamsergey@gmail.com",
    description="A small runner script which allows to run behaviour tests backed by proxy",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yamsergey/rikki",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    cmdclass={
        'verify': VerifyVersionCommand,
    },
    entry_points={
        'console_scripts': [
            "rikki = rikki.tools.main:rikki"
        ]
    },
    install_requires=[
        "allure-behave>=2.8.5",
        "allure-python-commons>=2.8.5",
        "Appium-Python-Client>=0.47",
        "behave>=1.2.6",
        "mitmproxy>=4.0.4",
        "selenium>=3.141.0",
    ]
)
